@@ -8,6 +8,7 @@ import tryton.common as common
 from tryton.gui.window.nomodal import NoModal
 from tryton.common import TRYTON_ICON
 from tryton.common import RPCExecute, RPCException
+from tryton.common.widget_style import widget_class
 
 _ = gettext.gettext
 
@@ -22,6 +23,7 @@ class Widget(object):
         self.widget = None
         self.mnemonic_widget = None
         self.visible = True
+        self._readonly = False
 
     @property
     def field_name(self):
@@ -48,6 +50,9 @@ class Widget(object):
         self._focus_out()
 
     def _readonly_set(self, readonly):
+        self._readonly = readonly
+
+    def _required_set(self, required):
         pass
 
     def _invisible_widget(self):
@@ -92,14 +97,22 @@ class Widget(object):
         if not field:
             self._readonly_set(self.attrs.get('readonly', True))
             self.invisible_set(self.attrs.get('invisible', False))
+            self._required_set(False)
             return
-        readonly = self.attrs.get('readonly',
-            field.get_state_attrs(record).get('readonly', False))
+        states = field.get_state_attrs(record)
+        readonly = self.attrs.get('readonly', states.get('readonly', False))
         if self.view.screen.readonly:
             readonly = True
         self._readonly_set(readonly)
-        self.invisible_set(self.attrs.get('invisible',
-            field.get_state_attrs(record).get('invisible', False)))
+        widget_class(self.widget, 'readonly', readonly)
+        self._required_set(not readonly and states.get('required', False))
+        widget_class(
+            self.widget, 'required',
+            not readonly and states.get('required', False))
+        invalid = states.get('invalid', False)
+        widget_class(self.widget, 'invalid', not readonly and invalid)
+        self.invisible_set(self.attrs.get(
+                'invisible', states.get('invisible', False)))
 
     def set_value(self, record, field):
         pass
@@ -107,7 +120,7 @@ class Widget(object):
 
 class TranslateDialog(NoModal):
 
-    def __init__(self, widget, languages):
+    def __init__(self, widget, languages, readonly):
         NoModal.__init__(self)
         self.widget = widget
         self.win = gtk.Dialog(_('Translation'), self.parent,
@@ -164,12 +177,13 @@ class TranslateDialog(NoModal):
             label.set_mnemonic_widget(widget)
             self.widget.translate_widget_set(widget, fuzzy_value)
             self.widget.translate_widget_set_readonly(widget, True)
-            yopt = 0
+            yopt = None
             if self.widget.expand:
-                yopt |= gtk.EXPAND | gtk.FILL
+                yopt = gtk.EXPAND | gtk.FILL
             table.attach(widget, 1, 2, i, i + 1, yoptions=yopt)
             editing = gtk.CheckButton()
             editing.connect('toggled', self.editing_toggled, widget)
+            editing.props.sensitive = not readonly
             tooltips.set_tip(editing, _('Edit'))
             table.attach(editing, 2, 3, i, i + 1, xoptions=gtk.FILL)
             fuzzy = gtk.CheckButton()
@@ -245,7 +259,7 @@ class TranslateMixin:
         self.view.set_value()
         if self.record.id < 0 or self.record.modified:
             common.message(
-                _('You need to save the record before adding translations!'))
+                _('You need to save the record before adding translations.'))
             return
 
         try:
@@ -256,7 +270,7 @@ class TranslateMixin:
             return
 
         if not lang_ids:
-            common.message(_('No other language available!'))
+            common.message(_('No other language available.'))
             return
         try:
             languages = RPCExecute('model', 'ir.lang', 'read', lang_ids,
@@ -264,7 +278,7 @@ class TranslateMixin:
         except RPCException:
             return
 
-        TranslateDialog(self, languages)
+        TranslateDialog(self, languages, self._readonly)
 
     def translate_widget(self):
         raise NotImplemented
